@@ -9,10 +9,16 @@ import sys
 import time
 from collections.abc import Iterator
 from contextlib import closing
-from typing import Optional
+from typing import Optional, Union
+if sys.version_info >= (3, 10):
+    from typing import TypeAlias
+else:
+    from typing_extensions import TypeAlias
 
 logger = logging.getLogger(__name__)
 
+StrPath: TypeAlias = Union[str, os.PathLike[str]]
+FileDescriptorOrPath: TypeAlias = Union[int, bytes, os.PathLike[bytes], StrPath]
 
 def get_public_ip() -> str:
     """Get the IP that this machine uses to contact the internet.
@@ -63,6 +69,24 @@ def unix_now_ms() -> float:
     """Return the current time in milliseconds since the Unix epoch."""
     return time.time() * 1000
 
+def unix_seconds_to_timestamp(timestamp: float) -> str:
+    """
+    Convert a time in seconds since the Unix epoch to an ISO 8601 string.
+    """
+    return datetime.datetime.fromtimestamp(timestamp, tz=datetime.timezone.utc).isoformat()
+
+def unix_seconds_to_local_time(timestamp: float) -> datetime.datetime:
+    """
+    Returns a local time corresponding to the given number of seconds since the Unix epoch.
+    """
+    return datetime.datetime.fromtimestamp(timestamp, tz=datetime.timezone.utc).astimezone()
+
+def seconds_to_duration(time_difference: float) -> str:
+    """
+    Convert a time difference in seconds to an ISO 8601 duration string.
+    """
+    return f"PT{time_difference:.3f}S"
+
 
 def slow_down(seconds: float) -> float:
     """
