@@ -86,3 +86,14 @@ def add_wdl_options(parser: ArgumentParser, suppress: bool = True) -> None:
         default=None,
         help=suppress_help or "Keep and return all call outputs as workflow outputs"
     )
+
+    strict_arguments = ["--wdlStrict"] + (
+        ["--strict"] if not suppress else []
+    )
+    parser.add_argument(
+        *strict_arguments,
+        dest="strict",
+        type=strtobool,
+        default=False,
+        help=suppress_help or "Exit runner if workflow has any lint warnings"
+    )
\ No newline at end of file
