@@ -10,13 +10,13 @@ from io import BytesIO
 from typing import Any, Optional, cast
 from urllib.parse import urldefrag, urljoin, urlparse
 
-import requests
 import ruamel.yaml
 import schema_salad
 from configargparse import ArgumentParser
 from wes_client.util import WESClient  # type: ignore
 from wes_client.util import wes_reponse as wes_response
 
+from toil.lib.web import web_session
 from toil.wdl.utils import get_version as get_wdl_version
 
 """
@@ -117,7 +117,7 @@ class WESClientWithWorkflowEngineParameters(WESClient):  # type: ignore
         proto, host = endpoint.split("://")
         super().__init__(
             {
-                # TODO: use the auth argument in requests.post so we don't need to encode it ourselves
+                # TODO: use the auth argument in requests' post so we don't need to encode it ourselves
                 "auth": (
                     {
                         "Authorization": "Basic "
@@ -144,7 +144,8 @@ class WESClientWithWorkflowEngineParameters(WESClient):  # type: ignore
             return "3.8"
         elif extension == "cwl":
             with open(workflow_file) as f:
-                return str(ruamel.yaml.safe_load(f)["cwlVersion"])
+                yaml = ruamel.yaml.YAML(typ='safe', pure=True)
+                return str(yaml.load(f)["cwlVersion"])
         elif extension == "wdl":
             with open(workflow_file) as f:
                 return get_wdl_version(f)
@@ -314,7 +315,7 @@ class WESClientWithWorkflowEngineParameters(WESClient):  # type: ignore
         data, files = self.build_wes_request(
             workflow_file, workflow_params_file, attachments, workflow_engine_parameters
         )
-        post_result = requests.post(
+        post_result = web_session.post(
             urljoin(f"{self.proto}://{self.host}", "/ga4gh/wes/v1/runs"),
             data=data,
             files=files,
