@@ -11,43 +11,42 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-import os
+from pathlib import Path
 
-from toil.job import Job
-from toil.test import ToilTest, get_temp_file
+from toil.job import Job, Promise
 from toil.test.src.jobTest import fn1Test
 
 
-class JobEncapsulationTest(ToilTest):
+class TestJobEncapsulation:
     """Tests testing the EncapsulationJob class."""
 
-    def testEncapsulation(self):
+    def testEncapsulation(self, tmp_path: Path) -> None:
         """
         Tests the Job.encapsulation method, which uses the EncapsulationJob
         class.
         """
         # Temporary file
-        outFile = get_temp_file(rootDir=self._createTempDir())
-        try:
-            # Encapsulate a job graph
-            a = Job.wrapJobFn(encapsulatedJobFn, "A", outFile, name="a")
-            a = a.encapsulate(name="a-encap")
-            # Now add children/follow to the encapsulated graph
-            d = Job.wrapFn(fn1Test, a.rv(), outFile, name="d")
-            e = Job.wrapFn(fn1Test, d.rv(), outFile, name="e")
-            a.addChild(d)
-            a.addFollowOn(e)
-            # Create the runner for the workflow.
-            options = Job.Runner.getDefaultOptions(self._getTestJobStorePath())
-            options.logLevel = "INFO"
-            # Run the workflow, the return value being the number of failed jobs
-            Job.Runner.startToil(a, options)
-            # Check output
-            self.assertEqual(open(outFile).readline(), "ABCDE")
-        finally:
-            os.remove(outFile)
+        rootDir = tmp_path / "out"
+        rootDir.mkdir()
+        outFile = rootDir / "out"
+        # Encapsulate a job graph
+        a = Job.wrapJobFn(encapsulatedJobFn, "A", outFile, name="a").encapsulate(
+            name="a-encap"
+        )
+        # Now add children/follow to the encapsulated graph
+        d = Job.wrapFn(fn1Test, a.rv(), outFile, name="d")
+        e = Job.wrapFn(fn1Test, d.rv(), outFile, name="e")
+        a.addChild(d)
+        a.addFollowOn(e)
+        # Create the runner for the workflow.
+        options = Job.Runner.getDefaultOptions(tmp_path / "jobstore")
+        options.logLevel = "INFO"
+        # Run the workflow, the return value being the number of failed jobs
+        Job.Runner.startToil(a, options)
+        # Check output
+        assert outFile.read_text() == "ABCDE"
 
-    def testAddChildEncapsulate(self):
+    def testAddChildEncapsulate(self) -> None:
         """
         Make sure that the encapsulate child does not have two parents
         with unique roots.
@@ -56,14 +55,14 @@ class JobEncapsulationTest(ToilTest):
         a = Job.wrapFn(noOp)
         b = Job.wrapFn(noOp)
         a.addChild(b).encapsulate()
-        self.assertEqual(len(a.getRootJobs()), 1)
+        assert len(a.getRootJobs()) == 1
 
 
-def noOp():
+def noOp() -> None:
     pass
 
 
-def encapsulatedJobFn(job, string, outFile):
+def encapsulatedJobFn(job: Job, string: str, outFile: Path) -> Promise:
     a = job.addChildFn(fn1Test, string, outFile, name="inner-a")
     b = a.addFollowOnFn(fn1Test, a.rv(), outFile, name="inner-b")
     return b.rv()
